<?php

namespace Teknisa\EST\Controllers;

use Teknisa\EST\Services\InventarioRotativoService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;

class InveRotaValidations extends Listener {

    protected $kernel;
    protected $inveRotaService;
    const INVENTARIO_PENDENTE = "Inventário Rotativo Pendente";
    const DATASET_INVENTARIO_VALIDO = "inventario_valido";

    public function __construct(InventarioRotativoService $inveRotaService, Kernel $kernel){
        $this->inveRotaService = $inveRotaService;
        $this->kernel = $kernel;
    }

    /*
     * Método que intercepta toda requisição feita ao backend, realizando
     * a verificação de inventário rotativo de acordo com os parâmetros
     * enviados. Caso a tela esteja parametrizada para verificar estoque
     * e haja inventário rotativo pendente, uma exceção é lançada e tratada
     * nos devidos métodos do frontend, impedindo, assim, o continuamento
     * das funcionalidades normais da tela.
     */
    public function preDispatch(Request $request) {
        $headers = $this->kernel->getHttpRequest()->getHeaders();
        $validaEstoque = $headers->get("ValidaEstoque");
        if ($validaEstoque == "true") {
            $cdfilial = $headers->get("Filial");
            if ($cdfilial != "false") {
                $isValid = $this->inveRotaService->validateInventarioRotativo($cdfilial);
            }
            else {
                $isValid = $this->inveRotaService->validateInventarioRotativo();
            }

            if (!$isValid) {
                throw new \Exception(self::INVENTARIO_PENDENTE);
            }
        }
    }
    
    
    public function newInveRotaValidation(Request $request, Response $response) {
        $headers = $this->kernel->getHttpRequest()->getHeaders();
        $validaEstoque = $headers->get("ValidaEstoque");
        if ($validaEstoque == "true") {
            $cdfilial = $headers->get("Filial");
            if ($cdfilial != "false") {
                $isValid = $this->inveRotaService->validateInventarioRotativo($cdfilial);
            }
            else {
                $isValid = $this->inveRotaService->validateInventarioRotativo();
            }

            if (!$isValid) {
                throw new \Exception(self::INVENTARIO_PENDENTE);
            }
        }
        $response->addDataSet(new DataSet(self::DATASET_INVENTARIO_VALIDO));
    }

    /*
     * Método de validação chamado pelo service no frontend.
     * Como o método de preDispatch vai verificar o inventário rotativo,
     * este método não seria chamado caso houvesse alguma pendência.
     * Como não há a real necessidade de repetir a validação neste método,
     * somente é adicionado um DataSet para indiciar que a verificação foi
     * realizada com sucesso.
     */
    public function validateInventarioRotativo(Request\Row $request, Response $response) {
        $response->addDataSet(new DataSet(self::DATASET_INVENTARIO_VALIDO));
    }
}