<?php
namespace Teknisa\EST\Services;

use Teknisa\EST\Factories\Est53000ParametrosDeEstoqueFactory;

class Est53000ParametrosDeEstoqueService {
    #default service
    protected $factory;

    /* **************************************************
     * Construtor do Service.
     *
     * @param Est53000ParametrosDeEstoqueFactory $factory
     */
    public function __construct(Est53000ParametrosDeEstoqueFactory $factory) {
        #initializes default service
        $this->factory = $factory;
    }

    public function buildData($factoryData, $singleRow = false) {
        $data = array();
        if($singleRow) {
            if(count($factoryData))
                foreach($factoryData[0] as $column => $value) {
                    $data[$column] = $value;
                }
        } else {
            foreach($factoryData as $factoryRow) {
                $row = array();
                foreach($factoryRow as $column => $value) {
                    $row[$column] = $value;
                }
                $data[] = $row;
            }
        }

        return $data;
    }

    public function deleteAllDays($CDFILIAL, $NRORG) {
        #throws exception in case of invalid parameters
        if(is_null($CDFILIAL) || is_null($NRORG) || $NRORG == 0)
            throw new \Exception("Erro Catastrófico! Erro na requisição dos parâmetros de unidade!", 1);

        #requests day deletion
        $this->factory->deleteAllDays($CDFILIAL, $NRORG);
    }

    public function getDataForGeral($NRORG) {
        #requests unidade tab data
        $factoryReturn = $this->factory->getDataForGeral($NRORG);

        #checks if something was found, throws exception otherwise
        if(count($factoryReturn) != 1){
            $this->factory->newParamestoque();
            $factoryReturn = $this->factory->getDataForGeral($NRORG);
            //throw new \Exception("Erro Catastrófico! Erro na requisição dos parâmetros gerais!", 1);
        }

        #builds data
        $data = $this->buildData($factoryReturn, true);

        #returns data
        return $data;
    }

    public function getDataForProdutoField($NRORG) {
        #requests unidade tab data
        $factoryReturn = $this->factory->getDataForProdutoField($NRORG);

        #builds data
        $data = $this->buildData($factoryReturn);

        #returns data
        return $data;
    }

    public function getDataForProdutoWidget($CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG, $CDARVPROD = false, $IDCURVAABC = false) {
        if(is_null($CDALMOXARIFE) || is_null($CDLOCALESTOQ) || is_null($CDFILIAL))
            throw new \Exception("Erro Catastrófico! Erro na requisição dos produtos da CurvaABC!", 1);

        #requests unidade tab data
        $factoryReturn = $this->factory->getDataForProdutoWidget($CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG, $CDARVPROD, $IDCURVAABC);

        $singleRow = ($CDARVPROD && $IDCURVAABC);

        #builds data
        $data = $this->buildData($factoryReturn, $singleRow);

        #returns data
        return $data;
    }

    public function validateProductInsertion($CDPRODUTO, $CDARVPROD, $IDCURVAABC, $CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG) {
        if(is_null($CDARVPROD) || is_null($IDCURVAABC))
            throw new \Exception("Erro Catastrófico! Erro na requisição de validação de inserção do produto!", 1);

        $data = $this->factory->getProductRequisitions($CDPRODUTO, $CDFILIAL, $NRORG);

        if(count($data) > 0)
            throw new \Exception("Produto não deve possuir composição.", 1);

        $data = $this->getDataForProdutoWidget($CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG, $CDARVPROD, $IDCURVAABC);

        if(count($data) > 0)
            throw new \Exception("Produto já cadastrado.", 1);

        return true;
    }

    public function saveProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG) {
        if(is_null($CDFILIAL) || is_null($CDALMOXARIFE) || is_null($CDLOCALESTOQ) || is_null($IDCURVAABC) || is_null($CDPRODUTO) || is_null($NRORG))
            throw new \Exception("Erro locais de estoque inválidos!", 1);

        $this->factory->saveProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG);
    }
    
    public function validateStockControll($row){
        
        if(isset($row['old_IDUTILALMOX']) && isset($row['old_IDUTILLCESTQ']) && isset($row['old_IDUTILLOTE'])){
            if( $row['old_IDUTILALMOX'] != $row['IDUTILALMOX']           ||
                $row['old_IDUTILLCESTQ'] != $row['IDUTILLCESTQ'] ||
                $row['old_IDUTILLOTE'] != $row['IDUTILLOTE']                ){
                    
                $result = $this->factory->validateStockControll($row['old_IDUTILALMOX'], $row['old_IDUTILLCESTQ'], $row['old_IDUTILLOTE'], $row['CDFILIAL']);
                if(count($result) > 0)
                    throw new \Exception("Operação Bloqueada! Alteração requer que o estoque seja zerado. É recomendado que antes de zerer o estoque seja emitida a posição de estoque atual da filial para posterior implantação.", 1);
            }
        }

    }

    public function updateProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG,$IDCURVAABC_OLD) {
        try{
            $this->factory->updateProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG, $IDCURVAABC_OLD);
        }catch(\Exception $e) {
            $pos = strpos($e->getMessage(), 'ORA-00001');
            if($pos===false){
                throw $e;
            }
            else{
                throw new \Exception("Produto já cadastrado.", 1);
            }
        }
    }

    public function deleteProdutosCurvaAbc($row) {
        $this->factory->deleteProdutosCurvaAbc($row);
    }

    public function saveGeral($row) {
        $this->factory->saveGeral($row);
        $this->factory->callProcedureJobMovimentacao();
    }

    public function saveUnidade($row) {
        $row['VRPETOLEINVROT'] = isset($row['VRPETOLEINVROT']) ? $row['VRPETOLEINVROT'] : null;

        $this->factory->saveUnidade($row);
        
        if($row['IDCTRLESTQ'] !== 'N'){
            $this->setPadroesEstoque($row['CDFILIAL']);
        }
        $this->insertParamplaneja($row['CDFILIAL']);
    }
    
    public function insertParamplaneja($filial){
        $paramPlaneja = $this->factory->getParamplaneja($filial);
        if(!$paramPlaneja)
            $this->factory->insertParamplaneja($filial);
    }
    
    public function setPadroesEstoque($filial){
        $this->factory->insertAlmoxPadrao($filial);
        $this->factory->insertLocalPadrao($filial);
        $lotes = $this->factory->getLotes($filial);
        if(!$lotes)
            $this->factory->insertLotesPadrao($filial);
    }

    public function saveParametrosCurvaAbc($row) {
        $this->factory->saveParametrosCurvaAbc($row);
    }

    public function getDataForUnidade($CDFILIAL, $NRORG) {
        #requests unidade tab data
        $factoryReturn = $this->factory->getDataForUnidade($CDFILIAL, $NRORG);

        #checks if something was found, throws exception otherwise
        if(count($factoryReturn) != 1)
            throw new \Exception("Parametros da unidade $CDFILIAL não encontrados.", 1);

        #builds data
        $data = $this->buildData($factoryReturn, true);

        #returns data
        return $data;
    }
}