<?php
namespace Estoque\Util;

use \Geral\Entities\Paramestoque;
use \Geral\Entities\Paramgeral;
use \Geral\Entities\Paramfilial;
use \Planejamento\Entities\Paramplaneja;
use \Geral\Util\Ambiente;

class Parametros
{
    /** @var Paramestoque */
    protected $paramEstoque;
    /** @var Paramfilial */
    protected $paramFilial;
    /** @var Paramgeral */
    protected $paramGeral;
    /** @var Paramplaneja */
    protected $paramPlaneja;

    /** @var Parametros */
    private static $instancia = null;
	
    protected function __construct($filial = null) {
        if (empty($filial) === true) {
            $filial = Ambiente::getCdFilial();
        }
        $params = array(Ambiente::getNrorg(), $filial);
        $this->paramGeral = Paramgeral::getObjectByWhere("NRORG = ?", array(Ambiente::getNrorg()));
        $this->paramFilial = Paramfilial::getObjectByWhere("NRORG = ? AND CDFILIAL = ?", $params);
        $this->paramEstoque = Paramestoque::getObjectByWhere("NRORG = ?", array(Ambiente::getNrorg()));
        $this->paramPlaneja = Paramplaneja::getObjectByWhere("NRORG = ? AND CDFILIAL = ?", $params);
    }

    public static function getInstancia($filial = null) {
        if(self::$instancia === null) {
            self::$instancia = new self($filial);
        }

        return self::$instancia;
    }

    public static function getAlmoxarifado() {
        $paramFilial = self::getInstancia()->paramFilial;
        if ($paramFilial->getIdutilalmox() === 'S') {
            return self::getInstancia()->paramFilial->getCdalmoxarife() ? : ' ';
        } else {
            return \Geral\Entities\Almoxarifado::ALMOXARIFADO_PADRAO;
        }
    }

    public static function getLocalEstoque() {
        $paramFilial = self::getInstancia()->paramFilial;
        if ($paramFilial->getIdutillcestq() === 'S') {
            return $paramFilial->getCdlocalestoq() ? : ' ';
        } else {
            return \Geral\Entities\Localestoque::LOCALESTOQUE_PADRAO;
        }
    }

    public static function getNumeroLote() {
        return \Geral\Entities\Lote::LOTEPADRAO;
    }

    public static function getNumeroSubLote() {
        return \Geral\Entities\Lote::SUBLOTEPADRAO;
    }

    public static function getOrdemEvolucaoAjuste() {
        return self::getInstancia()->paramFilial->getIdordajuinve();
    }

    public static function getOrdemImplantacao() {
        return self::getInstancia()->paramFilial->getIdordimplant();
    }
    
    public static function getIdgeraretaju(){
        return self::getInstancia()->paramFilial->getIdgeraretaju();
    }

    /**
     * @static
     * @return string
     */
    public static function utilDataProcessamento() {
        return self::getInstancia()->paramGeral->getIdutildtproc();
    }

    /**
     * @static
     * @return string
     */
    public static function getDataProcessamento($filial = null) {
        if (
            self::getInstancia()->paramGeral->getIdutildtproc() === 'S' &&
            self::getInstancia($filial)->paramFilial->getDtprocessa() !== ''
        ) {
            if(self::getInstancia($filial)->paramFilial->getDtprocessa())
               return self::getInstancia($filial)->paramFilial->getDtprocessa()->format('d/m/Y');
            else
               return date('d/m/Y');
        }
        return date('d/m/Y');
    }

    public static function getTipoCusto() {
        /** @var $tipocusto \Geral\Entities\Tipocusto */
        $tipocusto = self::getInstancia()->paramFilial->getTipcustest();
        return ($tipocusto && $tipocusto->getCdtipocusto()) ? $tipocusto->getCdtipocusto() : 3;
    }

    public static function getLancamentoDataMenor() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getIdlancdtmen()?: $instancia->paramFilial->getIdlancdtmenfil();
    }
    
    public static function getLancamentoDataMaior() {
    	$instancia = self::getInstancia();
    	return $instancia->paramEstoque->getIdlancdtmai()?: $instancia->paramFilial->getIdlancdtmaifil();
    }

    public static function getDiasEntradaMenor() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getNrdiasentr()?: $instancia->paramFilial->getNrdiasentrfil();
    }

    public static function getDiasSaidaMenor() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getNrdiassaid()?: $instancia->paramFilial->getNrdiassaidfil();
    }

    public static function getDiasAjusteMenor() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getNrdiasajus()?: $instancia->paramFilial->getNrdiasajusfil();
    }

    public static function getDiasEntradaMaior() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getNrdiasentrpd()?: $instancia->paramFilial->getNrdiasentrpdfil();
    }

    public static function getDiasSaidaMaior() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getNrdiassaidpd()?: $instancia->paramFilial->getNrdiassaidpdfil();
    }

    public static function getDiasAjusteMaior() {
        $instancia = self::getInstancia();
        return $instancia->paramEstoque->getNrdiasajuspd()? : $instancia->paramFilial->getNrdiasajuspdfil();
    }

    public static function getUtilizaLote() {
        return self::getInstancia()->paramFilial->getIdutillote();
    }

    public static function getBloqueioSemRetirada() {
        return self::getInstancia()->paramFilial->getIdblofertnev();
    }

    public static function getBloqueioSemNF() {
        return self::getInstancia()->paramFilial->getIdblofechmes();
    }

    public static function getBloqueiaFechamentoDia() {
    	return self::getInstancia()->paramFilial->getIdbloqfecdia();
    }
    
    public static function getBloqueioFechamentoSaidaNF() {
    	return self::getInstancia()->paramFilial->getIdbloqfechnf();
    }
    
    public static function getDiaFechamentoMes() {
        return self::getInstancia()->paramFilial->getNrdiafechmes();
    }

    public static function getDiaBloqueiaLancamento() {
    	return self::getInstancia()->paramFilial->getNrdiabloqest();
    }
    
    public static function getConsideraDiaNaoUtilFech() {
        return self::getInstancia()->paramFilial->getIdconsdiautfec();
    }
    
    public static function getUtilizaAlmoxarifado() {
        return self::getInstancia()->paramFilial->getIdutilalmox();
    }

    public static function getUtilizaLocalizacao() {
        return self::getInstancia()->paramFilial->getIdutillcestq();
    }

    public static function getPercentualInventarioRotativo() {
        return self::getInstancia()->paramFilial->getVrpercinvrot();
    }

    public static function getGeraInventarioRotativo() {
        return self::getInstancia()->paramFilial->getIdinvrotest();
    }

    public static function getInventarioRotativoRequisicao() {
        return self::getInstancia()->paramFilial->getIdinverotreq();
    }

    public static function getCalculoReceitaBruta() {
        return self::getInstancia()->paramFilial->getIdcalcrecbruta();
    }

    public static function getInventarioEstoqueCego() {
        return self::getInstancia()->paramFilial->getIdinvestcego();
    }

    public static function getTipoInventarioViradaMes() {
        return self::getInstancia()->paramFilial->getIdinvrotfmes();
    }

    public static function getInventarioRotativoDiariamente() {
        return self::getInstancia()->paramFilial->getIdinvrotdia();
    }

    public static function getPercentualCurvaA() {
        return self::getInstancia()->paramGeral->getVrpecurvaa();
    }

    public static function getPercentualCurvaB() {
        return self::getInstancia()->paramGeral->getVrpecurvab();
    }

    public static function getPercentualCurvaC() {
        return self::getInstancia()->paramGeral->getVrpecurvac();
    }

    public static function getObrigaEmissaoRE() {
        return self::getInstancia()->paramFilial->getIdconsrmcfdia();
    }

    public static function getConsisteRMC() {
        return self::getInstancia()->paramFilial->getIdconsrmcfdia();
    }

    public static function getControlaEstoque() {
        return self::getInstancia()->paramFilial->getIdctrlestq();
    }

    public static function getSistemaDos() {
        return self::getInstancia()->paramGeral->getIdsisdos();
    }

    public static function getVerificaEfetivos() {
        return self::getInstancia()->paramPlaneja->getIdefetcardia();
    }

    public static function getGeraRetiradaPorAjuste() {
        return self::getInstancia()->paramFilial->getIdgeraretaju();
    }
    
    public static function getAlteraDataFechamento() {
    	return self::getInstancia()->paramEstoque->getIdaltdtfecdia();
    }

    public static function getPermiteProdInativo()
    {
        return self::getInstancia()->paramFilial->getIdperproinat();
    }

}