<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 19/05/14
 * Time: 10:08
 */
namespace Folha\Logic\ObrigacaoLegal\RAIS;

use HCM\Util\DateUtil;

class TratamentoRais extends \FPARegistro
{

    const PARTICIPA_PAT = 1;

    /**
     * Acrescenta a quantidade de zeros ao inicio da variável
     * @param $variavel
     * @param $qnt
     * @return string
     */
    public function zeroInicial($variavel, $qnt)
    {
        if(is_null($variavel)) {
            $variavel = '0';
        }

        $var = str_pad($variavel, $qnt, '0', STR_PAD_LEFT);
        return $var;

    }

    /**
     * Acrescenta a quantidade de zeros ao final da variável
     * @param $variavel
     * @param $qnt
     * @return string
     */
    public function zeroFinal($variavel, $qnt)
    {
        if(is_null($variavel))
            $variavel = '0';

        $var = str_pad($variavel, $qnt, '0', STR_PAD_RIGHT);

        return ($var);
    }

    /**
     * Acrescenta a quantidade de espaços ao inicio da variável
     * @param $variavel
     * @param $qnt
     * @return string
     */
    public function espacoInicial($variavel, $qnt)
    {
        if(is_null($variavel))
            $variavel = ' ';
        else{
            $variavel = self::_trataDadoRais($variavel);
        }

        return (str_pad($variavel, $qnt, ' ', STR_PAD_LEFT));
    }

    /**
     * Acrescenta a quantidade de espaços ao final da variável
     * @param $variavel
     * @param $qnt
     * @return string
     */
    public function espacoFinal($variavel, $qnt)
    {
        if(is_null($variavel))
            $variavel = ' ';
        else{
            $variavel = self::_trataDadoRais($variavel);
        }

        return (str_pad($variavel, $qnt, ' ', STR_PAD_RIGHT));
    }

    public function tratarData($data, $formato, $tam = 8)
    {

        if($data){
            if(strlen($data) === $tam ){
               return $data;
            }
            if(!($data instanceof \DateTime)){

                list($d, $m, $a, $r) = mb_split('[-./: ]', $data, 4);

                if(checkdate($m, $d, $a)){
                    $data = DateUtil::getDataDeString("$d/$m/$a");
                    $data = $data->format($formato);
                }

            }else{
                $data->format($formato);
            }
        }
        else{
            $data = '0';
        }
        return ($data)?$data:str_pad($data, $tam, '0');
    }

    //VERIFICAÇÃO RAIS TIPO 01

    public function verificaCNPJ($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("CNPJ não informado para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaRazaoSocial($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Razão Social não informado para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaEndereco($logradouro, $numero, $cep, $cdmunicipio, $uf, $estrutura)
    {
        try{
            if(empty($logradouro)){
                throw new \Exception("Logradouro não informado para a estrutura $estrutura.");
            }
            if(empty($numero)){
                throw new \Exception("Número(Endereço) não informado para a estrutura $estrutura.");
            }
            if(empty($cep)){
                throw new \Exception("CEP não informado para a estrutura $estrutura.");
            }
            if(empty($cdmunicipio)){
                throw new \Exception("Municipio não informado para a estrutura $estrutura.");
            }
            if(empty($uf)){
                throw new \Exception("País não informado para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaEmail($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("E-mail não informado para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaCNAE($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Atividade Econômica não informada para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaNaturezaJuridica($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Natureza Jurídica não informada para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaIdentificacao($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Tipo de Inscrição não informada no filtro.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaPorte($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Porte da empresa não informada para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaSimples($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Opção por simples não informada para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    public function verificaPat($variavel, $estrutura)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Opção por participar do PAT não informada para a estrutura $estrutura.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    //VERIFICAÇÃO RAIS TIPO 02

    /**
     * Verifica se o PIS/PASEP foi informado
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaPispasep($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Pis/Pasep não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o nome foi informado
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaNome($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Nome não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se a data de nascimento foi informada
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaDtnascimento($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Data de nascimento não informada para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o há valor de nacionalidade, se não for Brasileiro, verifica se o campo ano de chegada está preenchida
     * @param $variavel
     * @param $anoChegada
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaNacionalidade($variavel, $anoChegada, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Nacionalidade não informada no vínculo $vinculo.");
            }elseif(($variavel != 10) && empty($anoChegada)){
                throw new \Exception("Será necessário preencher a data de chegada ao país do vínculo $vinculo para continuar.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }


    /**
     * Verifica se o grau de instrução foi informado
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaGrauinstrucao($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Grau de instrução não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o cpf foi informado
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaCpf($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("CPF não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se a data de admissão foi informada
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaDtadmissao($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Data de admissão não informada para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o salário foi informado
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaSalario($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Salário contratual não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o código do salário foi informada
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaCdsalario($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Código do salário não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se a quantidade de horas semanais foi informada
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaHorassemanais($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Quantidade de horas semanais não informada para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se a ocupação foi informada
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaOcupacao($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Ocupação não informada para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Vrifica se o tipo de vínculo foi informado.
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaVinculo($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Tipo de vínculo não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se a raça do vínculo foi informada.
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaRaca($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Cor/Raça não informada para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o sexo do vínculo foi informado.
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaSexo($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Sexo não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se o 13º salário do vínculo foi informado.
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verifica13salario($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Décimo terceiro salário não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se a contribuíção sindical do vínculo foi informado.
     * @param $variavel
     * @param $vinculo
     * @throws \Exception
     */
    public function verificaContribuicaoSindical($variavel, $vinculo)
    {
        try{
            if(empty($variavel)){
                throw new \Exception("Contribuição Sindical não informado para o vínculo $vinculo.");
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    public function verificaDtdemissao($variavel, $anoRais)
    {}

    public function trataTelefone($tel)
    {
        if (!is_null($tel)) {
            $b = preg_replace('/[^0-9]/', '', $tel);

            $tam = strlen($b);

            switch ($tam) {
                //99-9999-99999
                case(11):
                {
                    $ddd = substr($b, 0, 2);
                    $tel = substr($b, -9);
                    break;
                }
                //99-9999-9999
                case(10):
                {
                    $ddd = substr($b, 0, 2);
                    $tel = substr($b, -8);
                    break;
                }
                //9999-99999
                case(9):
                {
                    return $tel;
                    break;
                }
                //9999-9999
                case(8):
                {
                    break;
                }
                default:
                    {
                    if ($tam < 8) {
                        $tel = substr($b, -9);
                    } else {
                        $tel  = substr($b, -8);
                    }
                }
            }
        }else{
            $tel = '0';
        }

        $telefone = str_pad($tel, 9, '0', STR_PAD_LEFT);
        return $telefone;

    }

    /**
     * Contrato - Gera Registro.
     *
     * @param string $tpRegistro  Numero do Tipo de Registro.
     * @param string $tpInscricao Numero do Tipo de Inscricao.
     * @param string $nrInscricao Numero da Inscricao.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     *
     * ESTE MÉTODO SERÁ IMPLEMENTADO EM GERARAIS.PHP
     */
    protected function geraRegistro($tpRegistro, $tpInscricao, $nrInscricao){}

    public function validaValorPAT($valor, $pat)
    {
        if($pat == self::PARTICIPA_PAT ){
            $valor = self::zeroInicial($valor, 6);
        }else{
            $valor = self::zeroInicial(null, 6);
        }

        return $valor;
    }

}